--- src/global/mail_proto.h.orig	2011-06-06 16:12:20.000000000 +1200
+++ src/global/mail_proto.h	2011-06-06 16:11:09.000000000 +1200
@@ -208,6 +208,7 @@
 #define XCLIENT_PORT		"PORT"	/* client port */
 #define XCLIENT_PROTO		"PROTO"	/* client protocol */
 #define XCLIENT_HELO		"HELO"	/* client helo */
+#define XCLIENT_LOGIN           "LOGIN" /* authenticated client SASL login */
 
 #define XCLIENT_UNAVAILABLE	"[UNAVAILABLE]"	/* permanently unavailable */
 #define XCLIENT_TEMPORARY	"[TEMPUNAVAIL]"	/* temporarily unavailable */
--- src/smtpd/smtpd.c.orig	2011-06-06 14:23:39.000000000 +1200
+++ src/smtpd/smtpd.c	2011-06-06 15:59:31.000000000 +1200
@@ -1669,10 +1669,12 @@
     /* XCLIENT must not override its own access control. */
     if ((discard_mask & EHLO_MASK_XCLIENT) == 0)
 	if (xclient_allowed)
+            /* XXX - XCLIENT_LOGIN should be adversed only if(var_smtpd_sasl_enable) */
 	    ENQUEUE_FIX_REPLY(state, reply_buf, XCLIENT_CMD
 			      " " XCLIENT_NAME " " XCLIENT_ADDR
 			      " " XCLIENT_PROTO " " XCLIENT_HELO
-			      " " XCLIENT_REVERSE_NAME " " XCLIENT_PORT);
+			      " " XCLIENT_REVERSE_NAME " " XCLIENT_PORT
+                              " " XCLIENT_LOGIN);
     if ((discard_mask & EHLO_MASK_XFORWARD) == 0)
 	if (xforward_allowed)
 	    ENQUEUE_FIX_REPLY(state, reply_buf, XFORWARD_CMD
@@ -3414,6 +3416,9 @@
     };
     int     got_helo = 0;
     int     got_proto = 0;
+#ifdef USE_SASL_AUTH
+    int     got_sasl_login = 0;
+#endif
 
     /*
      * Sanity checks.
@@ -3606,6 +3611,20 @@
 	    got_proto = 1;
 	}
 
+#ifdef USE_SASL_AUTH
+        /*
+         * LOGIN=substitute SMTP client SASL login. Also set method as XCLIENT
+         */
+        else if (var_smtpd_sasl_enable && STREQ(attr_name, XCLIENT_LOGIN)) {
+            /*
+             * XXX may be here need some saninty check of attr_value
+             * but we trast to frontend
+             */
+            UPDATE_STR(state->sasl_username, attr_value);
+            UPDATE_STR(state->sasl_method, XCLIENT_CMD);
+            got_sasl_login = 1;
+        }
+#endif
 	/*
 	 * Unknown attribute name. Complain.
 	 */
@@ -3654,7 +3673,7 @@
 	state->protocol = mystrdup(MAIL_PROTO_SMTP);
     }
 #ifdef USE_SASL_AUTH
-    if (smtpd_sasl_is_active(state))
+    if (got_sasl_login == 0 && smtpd_sasl_is_active(state))
 	smtpd_sasl_auth_reset(state);
 #endif
     chat_reset(state, 0);
